@@ -73,11 +73,12 @@ Here is ASCII flow diagram for you::
  TitleQuery     ----|          ExportRequest          |--> SearchResult
  GenericQuery   ----|      ISBNValidationRequest      |       `- AlephRecord
  DocumentQuery  ----|                |                |
-                    |                |                |--> ISBNValidationResult
-                    V                |                |        - ISBN
-       Count/Search/ExportRequest    |                |
-                    |                |                |--> ExportResult
+ ICZQuery       ----|                |                |--> ISBNValidationResult
+                    |                |                |        - ISBN
                     V                |                |
+       Count/Search/ExportRequest    |                |--> ExportResult
+                    |                |                |
+                    |                |                |
                     |                |                |
                     V                |                |
                serialize()<----------'           deserialize()
@@ -105,11 +106,11 @@ Neat, isn't it?
 API
 ---
 """
-#= Imports ====================================================================
+# Imports =====================================================================
 from collections import namedtuple
 
+import isbn_validator
 
-import isbn
 import aleph
 import export
 import settings
@@ -117,7 +118,7 @@ from datastructures import *
 from datastructures import convertor
 
 
-#= Queries ====================================================================
+# Queries =====================================================================
 class _QueryTemplate:
     """
     This class is here to just save some effort by using common ancestor with
@@ -127,16 +128,13 @@ class _QueryTemplate:
     """
     def getSearchResult(self):
         records = []
-        for doc_id, library, base in self._getIDs():
-            xml = aleph.downloadMARCOAI(doc_id, library)
-
+        for xml in self._getXML():
             records.append(
                 AlephRecord(
-                    base,
-                    library,
-                    doc_id,
-                    xml,
-                    convertor.toEPublication(xml)
+                    base=self.base,
+                    library=settings.DEFAULT_LIBRARY,
+                    docNumber=convertor.getDocNumber(xml),
+                    xml=xml
                 )
             )
 
@@ -160,7 +158,9 @@ class GenericQuery(namedtuple("GenericQuery", ['base',
                     for details.
         phrase (str): What are you looking for.
         considerSimilar (bool): Don't use this, it usually doesn't work.
-        field (str): Which field you want to use for search. See :attr:`aleph.VALID_ALEPH_FIELDS` for list of valid bases.
+        field (str): Which field you want to use for search.
+                     See :attr:`aleph.VALID_ALEPH_FIELDS` for list of valid
+                     bases.
 
     For details of base/phrase/.. parameters, see :func:`aleph.searchInAleph`.
     All parameters also serves as properties.
@@ -187,29 +187,6 @@ class GenericQuery(namedtuple("GenericQuery", ['base',
         )["no_entries"]
 
 
-class ISBNQuery(namedtuple("ISBNQuery", ["ISBN", "base"]), _QueryTemplate):
-    """
-    Used to query Aleph to get books by ISBN.
-
-    Args:
-        ISBN (str): ISBN 10/13.
-        base (str, optional): If not set, :attr:`settings.ALEPH_DEFAULT_BASE`
-                              is used.
-
-    Note:
-        ISBN is not unique, so you can get back lot of books with same ISBN.
-        Some books also have two or more ISBNs.
-    """
-    def __new__(self, ISBN, base=settings.ALEPH_DEFAULT_BASE):
-        return super(ISBNQuery, self).__new__(self, ISBN, base)
-
-    def _getIDs(self):
-        return aleph.getISBNsIDs(self.ISBN, base=self.base)
-
-    def _getCount(self):
-        return aleph.getISBNCount(self.ISBN, base=self.base)
-
-
 class DocumentQuery(namedtuple("DocumentQuery", ["doc_id", "library"])):
     """
     Query Aleph when you know the Document ID.
@@ -251,10 +228,34 @@ class DocumentQuery(namedtuple("DocumentQuery", ["doc_id", "library"])):
         """
         try:
             self.getSearchResult()
-            return 1
         except aleph.DocumentNotFoundException:
             return 0
 
+        return 1
+
+
+class ISBNQuery(namedtuple("ISBNQuery", ["ISBN", "base"]), _QueryTemplate):
+    """
+    Used to query Aleph to get books by ISBN.
+
+    Args:
+        ISBN (str): ISBN 10/13.
+        base (str, optional): If not set, :attr:`settings.ALEPH_DEFAULT_BASE`
+                              is used.
+
+    Note:
+        ISBN is not unique, so you can get back lot of books with same ISBN.
+        Some books also have two or more ISBNs.
+    """
+    def __new__(self, ISBN, base=settings.ALEPH_DEFAULT_BASE):
+        return super(ISBNQuery, self).__new__(self, ISBN, base)
+
+    def _getXML(self):
+        return aleph.getISBNsXML(self.ISBN, base=self.base)
+
+    def _getCount(self):
+        return aleph.getISBNCount(self.ISBN, base=self.base)
+
 
 class AuthorQuery(namedtuple("AuthorQuery", ["author", "base"]),
                   _QueryTemplate):
@@ -263,15 +264,14 @@ class AuthorQuery(namedtuple("AuthorQuery", ["author", "base"]),
 
     Args:
         author (str): Author's name/lastname in UTF-8.
-        base (str, optional): If not set, :attr:`settings.ALEPH_DEFAULT_BASE` is
-                              used.
-
+        base (str, optional): If not set, :attr:`settings.ALEPH_DEFAULT_BASE`
+                              is used.
     """
     def __new__(self, author, base=settings.ALEPH_DEFAULT_BASE):
         return super(AuthorQuery, self).__new__(self, author, base)
 
-    def _getIDs(self):
-        return aleph.getAuthorsBooksIDs(self.author, base=self.base)
+    def _getXML(self):
+        return aleph.getAuthorsBooksXML(self.author, base=self.base)
 
     def _getCount(self):
         return aleph.getAuthorsBooksCount(self.author, base=self.base)
@@ -284,15 +284,14 @@ class PublisherQuery(namedtuple("PublisherQuery", ["publisher", "base"]),
 
     Args:
         publisher (str): Publisher's name in UTF-8.
-        base (str, optional): If not set, :attr:`settings.ALEPH_DEFAULT_BASE` is
-                              used.
-
+        base (str, optional): If not set, :attr:`settings.ALEPH_DEFAULT_BASE`
+                              is used.
     """
     def __new__(self, publisher, base=settings.ALEPH_DEFAULT_BASE):
         return super(PublisherQuery, self).__new__(self, publisher, base)
 
-    def _getIDs(self):
-        return aleph.getPublishersBooksIDs(self.publisher, base=self.base)
+    def _getXML(self):
+        return aleph.getPublishersBooksXML(self.publisher, base=self.base)
 
     def _getCount(self):
         return aleph.getPublishersBooksCount(self.publisher, base=self.base)
@@ -305,38 +304,58 @@ class TitleQuery(_QueryTemplate,
 
     Args:
         title (str): Book's title in UTF-8.
-        base (str, optional): If not set, :attr:`settings.ALEPH_DEFAULT_BASE` is
-                              used.
-
+        base (str, optional): If not set, :attr:`settings.ALEPH_DEFAULT_BASE`
+                              is used.
     """
     def __new__(self, title, base=settings.ALEPH_DEFAULT_BASE):
         return super(TitleQuery, self).__new__(self, title, base)
 
-    def _getIDs(self):
-        return aleph.getBooksTitleIDs(self.title, base=self.base)
+    def _getXML(self):
+        return aleph.getBooksTitleXML(self.title, base=self.base)
 
     def _getCount(self):
         return aleph.getBooksTitleCount(self.title, base=self.base)
 
-#= Variables ==================================================================
+
+class ICZQuery(_QueryTemplate, namedtuple("ICZQuery", ["icz", "base"])):
+    """
+    Used to query Aleph to get books by record's identification number `icz`.
+
+    Args:
+        icz (str): Identification number (``nkc20150003029`` for example).
+        base (str, optional): If not set, :attr:`settings.ALEPH_DEFAULT_BASE`
+                              is used.
+    """
+    def __new__(self, icz, base=settings.ALEPH_DEFAULT_BASE):
+        return super(ICZQuery, self).__new__(self, icz, base)
+
+    def _getXML(self):
+        return aleph.getICZBooksXML(self.icz, base=self.base)
+
+    def _getCount(self):
+        return aleph.getICZBooksCount(self.icz, base=self.base)
+
+
+# Variables ===================================================================
 QUERY_TYPES = [
     ISBNQuery,
     AuthorQuery,
     PublisherQuery,
     TitleQuery,
     GenericQuery,
-    DocumentQuery
+    DocumentQuery,
+    ICZQuery,
 ]
 
 REQUEST_TYPES = [
     SearchRequest,
     CountRequest,
     ExportRequest,
-    ISBNValidationRequest
+    ISBNValidationRequest,
 ]
 
 
-#= Interface for an external world ============================================
+# Interface for an external world =============================================
 def _iiOfAny(instance, classes):
     """
     Returns true, if `instance` is instance of any (_iiOfAny) of the `classes`.
@@ -368,8 +387,8 @@ def _iiOfAny(instance, classes):
     return any(map(lambda x: type(instance).__name__ == x.__name__, classes))
 
 
-#= Functions ==================================================================
-def reactToAMQPMessage(req, UUID):
+# Functions ===================================================================
+def reactToAMQPMessage(req, send_back):
     """
     React to given (AMQP) message.
 
@@ -385,7 +404,7 @@ def reactToAMQPMessage(req, UUID):
         >>> request
         SearchRequest(query=ISBNQuery(ISBN='80-251-0225-4', base='nkc'))
 
-        >>> response = aleph.reactToAMQPMessage(request, "UUID")
+        >>> response = aleph.reactToAMQPMessage(request, None)
 
         >>> response  # formated by hand for purposes of example
         SearchResult(
@@ -430,7 +449,10 @@ def reactToAMQPMessage(req, UUID):
     Args:
         req (Request class): Any of the Request class from
                           :class:`aleph.datastructures.requests`.
-        UUID (str): Unique ID of received message.
+        send_back (fn reference): Reference to function for responding. This is
+                  useful for progress monitoring for example. Function takes
+                  one parameter, which may be response structure/namedtuple, or
+                  string or whatever would be normally returned.
 
     Returns:
         Result class: Result of search in Aleph. \
@@ -456,7 +478,7 @@ def reactToAMQPMessage(req, UUID):
         if _iiOfAny(ISBN, ISBNQuery):
             ISBN = ISBN.ISBN
 
-        return ISBNValidationResult(isbn.is_valid_isbn(ISBN))
+        return ISBNValidationResult(isbn_validator.is_valid_isbn(ISBN))
 
     elif _iiOfAny(req, ExportRequest):
         export.exportEPublication(req.epublication)
