@@ -3,7 +3,7 @@
 #
 # Interpreter version: python 2.7
 #
-#= Imports ====================================================================
+# Imports =====================================================================
 """
 Aleph X-Service wrapper.
 
@@ -21,6 +21,7 @@ Lowlevel
 You can use this functions to access Aleph::
 
     searchInAleph(base, phrase, considerSimilar, field)
+    downloadRecords(search_result, [from_doc])
     getDocumentIDs(aleph_search_result, [number_of_docs])
     downloadMARCXML(doc_id, library)
     downloadMARCOAI(doc_id, base)
@@ -38,8 +39,9 @@ This dictionary can be later used as parameter to :func:`getDocumentIDs`
 function, which will give you list of :class:`DocumentID` named tuples.
 
 Note:
-    :py:func:`~collections.namedtuple` is used, because to access your document,
-    you don't need just `document ID` number, but also `library ID` string.
+    :py:func:`~collections.namedtuple` is used, because to access your
+    document, you don't need just `document ID` number, but also `library ID`
+    string.
 
 Depending on your system, there may be just only one accessible library, or
 multiple ones, and then you will be glad, that you get both of this
@@ -58,23 +60,43 @@ Lets look at some code::
 High-level
 ==========
 
-So far, there are only getter wrappers:
+XML wrappers
+************
+This wrappers returns full XML records from Aleph:
+
+    - :func:`getISBNsXML`
+    - :func:`getAuthorsBooksXML`
+    - :func:`getPublishersBooksXML`
+    - :func:`getBooksTitleXML`
+    - :func:`getICZBooksXML`
+
+ID wrappers
+***********
+There are wrappers, which returns ID's of matching document in Aleph:
 
     - :func:`getISBNsIDs`
     - :func:`getAuthorsBooksIDs`
     - :func:`getPublishersBooksIDs`
     - :func:`getBooksTitleIDs`
+    - :func:`getICZBooksIDs`
 
-And counting functions:
+You can theh download them using :func:`downloadMARCXML` or
+:func:`downloadMARCOAI`.
+
+Count wrappers
+**************
+Count wrappers returns just the number of records with given parameters are
+there in aleph.
 
     - :func:`getISBNCount`
     - :func:`getAuthorsBooksCount`
     - :func:`getPublishersBooksCount`
     - :func:`getBooksTitleCount`
+    - :func:`getICZBooksCount`
 
 Note:
-    Counting functions are by one request faster than just counting results from
-    standard getters. It is preferred to use them to reduce load to Aleph.
+    Counting functions are by one request faster than just counting results
+    from standard getters. It is preferred to use them to reduce load to Aleph.
 
 Other noteworthy properties
 ===========================
@@ -89,45 +111,47 @@ from collections import namedtuple
 from string import Template
 from urllib import quote_plus
 
-
 import dhtmlparser
 from httpkie import Downloader
 
-
 from settings import *
 
 
-#= Variables ==================================================================
+# Variables ===================================================================
 # String.Template() variable convention is used
-ALEPH_SEARCH_URL_TEMPLATE = "/X?op=find&request=$FIELD=$PHRASE&base=$BASE&adjacent=$SIMILAR"
-ALEPH_GET_SET_URL_TEMPLATE = "/X?op=ill_get_set&set_number=$SET_NUMBER&start_point=1&no_docs=$NUMBER_OF_DOCS"
-ALEPH_GET_DOC_URL_TEMPLATE = "/X?op=ill_get_doc&doc_number=$DOC_ID&library=$LIBRARY"
-ALEPH_GET_OAI_DOC_URL_TEMPLATE = "/X?op=find_doc&doc_num=$DOC_ID&base=$BASE"
+SEARCH_URL_TEMPLATE = "/X?op=find&request=$FIELD=$PHRASE&base=$BASE"
+SET_URL_TEMPLATE = "/X?op=ill_get_set&set_number=$SET_NUMBER" + \
+                   "&start_point=1&no_docs=$NUMBER_OF_DOCS"
+DOC_URL_TEMPLATE = "/X?op=ill_get_doc&doc_number=$DOC_ID&library=$LIBRARY"
+OAI_DOC_URL_TEMPLATE = "/X?op=find_doc&doc_num=$DOC_ID&base=$BASE"
+RECORD_URL_TEMPLATE = "/X?op=present&set_number=$SET_NUM&set_entry=$RECORD_NUM"
+
+MAX_RECORDS = 30
 
 
 VALID_ALEPH_FIELDS = [
-    "wrd",  # Všechny údaje
-    "wtl",  # Název
-    "wau",  # Autor (osoba, korporace)
-    "wkw",  # Předmět (klíčová slova)
-    "txt",  # Slova z obsahu (table of cont.)
-    "wpb",  # Nakladatel
-    "wpp",  # Místo vydání
-    "wyr",  # Rok vydání
-    "ssn",  # ISSN
-    "sbn",  # ISBN / ISMN
-    "isn",  # ISBN / ISMN / ISSN
-    "ob",   # Obsazení (hudební díla)
-    "wpf",  # Periodicita
-    "wpv",  # Kód země vydání
-    "wln",  # Kód jazyka dokumentu
-    "wlo",  # Kód jazyka originálu
-    "wtp",  # Druh dokumentu
-    "sg",   # Signatura
-    "bar",  # Čárový kód
-    "cnb",  # Číslo národní bibl.
-    "icz",  # Identifikační číslo
-    "sys",  # Systémové číslo
+    "wrd",
+    "wtl",
+    "wau",
+    "wkw",
+    "txt",
+    "wpb",
+    "wpp",
+    "wyr",
+    "ssn",
+    "sbn",
+    "isn",
+    "ob",
+    "wpf",
+    "wpv",
+    "wln",
+    "wlo",
+    "wtp",
+    "sg",
+    "bar",
+    "cnb",
+    "icz",
+    "sys",
     "wpk",
 ]
 """
@@ -159,7 +183,7 @@ VALID_ALEPH_FIELDS = [
 dhtmlparser.NONPAIR_TAGS = []  # used for parsing XML - see documentation
 
 
-#= Functions & objects ========================================================
+# Functions & objects =========================================================
 class AlephException(Exception):
     """
     Exception tree::
@@ -202,8 +226,8 @@ class DocumentID(namedtuple("DocumentID", ["id", "library", "base"])):
         id (int): ID of document.
         library (str): This can be different for each document, depend on your
                        system.
-        base (str): Default "``nkc``", but really depends on what bases you have
-                    defined in your Aleph server.
+        base (str): Default "``nkc``", but really depends on what bases you
+                    have defined in your Aleph server.
     """
     pass
 
@@ -332,7 +356,7 @@ def searchInAleph(base, phrase, considerSimilar, field):
     if field.lower() not in VALID_ALEPH_FIELDS:
         raise InvalidAlephFieldException("Unknown field '" + field + "'!")
 
-    param_url = Template(ALEPH_SEARCH_URL_TEMPLATE).substitute(
+    param_url = Template(SEARCH_URL_TEMPLATE).substitute(
         PHRASE=quote_plus(phrase),  # urlencode phrase
         BASE=base,
         FIELD=field,
@@ -364,6 +388,48 @@ def searchInAleph(base, phrase, considerSimilar, field):
         raise AlephException(result["error"])
 
 
+def downloadRecords(search_result, from_doc=1):
+    """
+    Download `MAX_RECORDS` documents from `search_result` starting from
+    `from_doc`.
+
+    Attr:
+        search_result (dict): returned from :func:`searchInAleph`.
+        from_doc (int, default 1): Start from document number `from_doc`.
+
+    Returns:
+        list: List of XML strings with documents in MARC OAI.
+    """
+    downer = Downloader()
+
+    if "set_number" not in search_result:
+        return []
+
+    # set numbers should be probably aligned to some length
+    set_number = str(search_result["set_number"])
+    if len(set_number) < 6:
+        set_number = (6 - len(set_number)) * "0" + set_number
+
+    # download all no_records
+    records = []
+    for cnt in range(search_result["no_records"]):
+        doc_number = from_doc + cnt
+
+        if cnt >= MAX_RECORDS or doc_number > search_result["no_records"]:
+            break
+
+        set_data = downer.download(
+            ALEPH_URL + Template(RECORD_URL_TEMPLATE).substitute(
+                SET_NUM=set_number,
+                RECORD_NUM=doc_number,
+            )
+        )
+
+        records.append(set_data)
+
+    return records
+
+
 def getDocumentIDs(aleph_search_result, number_of_docs=-1):
     """
     Get IDs, which can be used as parameters for other functions.
@@ -400,7 +466,7 @@ def getDocumentIDs(aleph_search_result, number_of_docs=-1):
 
     # download data about given set
     set_data = downer.download(
-        ALEPH_URL + Template(ALEPH_GET_SET_URL_TEMPLATE).substitute(
+        ALEPH_URL + Template(SET_URL_TEMPLATE).substitute(
             SET_NUMBER=set_number,
             NUMBER_OF_DOCS=number_of_docs,
         )
@@ -446,7 +512,7 @@ def getDocumentIDs(aleph_search_result, number_of_docs=-1):
     return ids
 
 
-def downloadMARCXML(doc_id, library):
+def downloadMARCXML(doc_id, library, base="nkc"):
     """
     Download MARC XML document with given `doc_id` from given `library`.
 
@@ -466,7 +532,7 @@ def downloadMARCXML(doc_id, library):
     downer = Downloader()
 
     data = downer.download(
-        ALEPH_URL + Template(ALEPH_GET_DOC_URL_TEMPLATE).substitute(
+        ALEPH_URL + Template(DOC_URL_TEMPLATE).substitute(
             DOC_ID=doc_id,
             LIBRARY=library
         )
@@ -477,24 +543,24 @@ def downloadMARCXML(doc_id, library):
     # check if there are any errors
     # bad library error
     error = dom.find("login")
-    if len(error) > 0:
-        error = error[0].find("error")
+    if error:
+        error_msg = error[0].find("error")
 
-        if len(error) > 0:
+        if error_msg:
             raise LibraryNotFoundException(
                 "Can't download document doc_id: '" + str(doc_id) + "' " +
                 "(probably bad library: '" + library + "')!\nMessage: " +
-                error.getContent()
+                "\n".join(map(lambda x: x.getContent(), error_msg))
             )
 
     # another error - document not found
     error = dom.find("ill-get-doc")
-    if len(error) > 0:
-        error = error[0].find("error")
+    if error:
+        error_msg = error[0].find("error")
 
-        if len(error) > 0:
+        if error_msg:
             raise DocumentNotFoundException(
-                error[0].getContent()
+                "\n".join(map(lambda x: x.getContent(), error_msg))
             )
 
     return data  # MARCxml of document with given doc_id
@@ -525,7 +591,7 @@ def downloadMARCOAI(doc_id, base):
     downer = Downloader()
 
     data = downer.download(
-        ALEPH_URL + Template(ALEPH_GET_OAI_DOC_URL_TEMPLATE).substitute(
+        ALEPH_URL + Template(OAI_DOC_URL_TEMPLATE).substitute(
             DOC_ID=doc_id,
             BASE=base
         )
@@ -549,6 +615,123 @@ def downloadMARCOAI(doc_id, base):
         )
 
 
+# High level API ==============================================================
+def getISBNsXML(isbn, base=ALEPH_DEFAULT_BASE):
+    """
+    Download full XML record for given `isbn` in `base`.
+
+    Args:
+        isbn (str): ISBN of the books you want to get.
+        base (str): Base on which will be search performed. Default
+                    :attr:`aleph.settings.ALEPH_DEFAULT_BASE`.
+
+    Returns:
+        list: List of strings with full **OAI** XML representation of the \
+              record.
+    """
+    return downloadRecords(
+        searchInAleph(
+            base,
+            isbn,
+            False,
+            "sbn"
+        )
+    )
+
+
+def getAuthorsBooksXML(author, base=ALEPH_DEFAULT_BASE):
+    """
+    Download full XML record for given `author` in `base`.
+
+    Args:
+        author (str): Name of the `author` of the books you want to get.
+        base (str): Base on which will be search performed. Default
+                    :attr:`aleph.settings.ALEPH_DEFAULT_BASE`.
+
+    Returns:
+        list: List of strings with full **OAI** XML representation of the \
+              record.
+    """
+    return downloadRecords(
+        searchInAleph(
+            base,
+            author,
+            False,
+            "wau"
+        )
+    )
+
+
+def getPublishersBooksXML(publisher, base=ALEPH_DEFAULT_BASE):
+    """
+    Download full XML record for given `publisher` in `base`.
+
+    Args:
+        publisher (str): Name of the `publisher` of the books you want to get.
+        base (str): Base on which will be search performed. Default
+                    :attr:`aleph.settings.ALEPH_DEFAULT_BASE`.
+
+    Returns:
+        list: List of strings with full **OAI** XML representation of the \
+              record.
+    """
+    return downloadRecords(
+        searchInAleph(
+            base,
+            publisher,
+            False,
+            "wpb"
+        )
+    )
+
+
+def getBooksTitleXML(title, base=ALEPH_DEFAULT_BASE):
+    """
+    Download full XML record for given `title` in `base`.
+
+    Args:
+        title (str): `title` of the books you want to get.
+        base (str): Base on which will be search performed. Default
+                    :attr:`aleph.settings.ALEPH_DEFAULT_BASE`.
+
+    Returns:
+        list: List of strings with full **OAI** XML representation of the \
+              record.
+    """
+    return downloadRecords(
+        searchInAleph(
+            base,
+            title,
+            False,
+            "wtl"
+        )
+    )
+
+
+def getICZBooksXML(icz, base=ALEPH_DEFAULT_BASE):
+    """
+    Download full XML record for given `icz` (identification number) in `base`.
+
+    Args:
+        icz (str): Identification number used to search Aleph.
+        base (str): Base on which will be search performed. Default
+                    :attr:`aleph.settings.ALEPH_DEFAULT_BASE`.
+
+    Returns:
+        list: List of strings with full **OAI** XML representation of the \
+              record.
+    """
+    return downloadRecords(
+        searchInAleph(
+            base,
+            icz,
+            False,
+            "icz"
+        )
+    )
+
+
+# ID getters ==================================================================
 def getISBNsIDs(isbn, base=ALEPH_DEFAULT_BASE):
     """
     Get list of :class:`DocumentID` objects of documents with given `isbn`.
@@ -612,6 +795,23 @@ def getBooksTitleIDs(title, base=ALEPH_DEFAULT_BASE):
     return getDocumentIDs(searchInAleph(base, title, False, "wtl"))
 
 
+def getICZBooksIDs(icz, base=ALEPH_DEFAULT_BASE):
+    """
+    Get list of :class:`DocumentID` objects of documents with given
+    `icz` (identification number).
+
+    Args:
+        icz (str): Identification number used to search Aleph.
+        base (str, optional): base on which will be search performed. Default
+                    :attr:`aleph.settings.ALEPH_DEFAULT_BASE`.
+
+    Returns:
+        list: of :class:`DocumentID` objects
+    """
+    return getDocumentIDs(searchInAleph(base, icz, False, "icz"))
+
+
+# Counters ====================================================================
 def getISBNCount(isbn, base=ALEPH_DEFAULT_BASE):
     """
     Get number of records in Aleph which match given `isbn`.
@@ -670,3 +870,18 @@ def getBooksTitleCount(title, base=ALEPH_DEFAULT_BASE):
         int: Number of matching documents in Aleph.
     """
     return searchInAleph(base, title, False, "wtl")["no_entries"]
+
+
+def getICZBooksCount(icz, base=ALEPH_DEFAULT_BASE):
+    """
+    Get number of records in Aleph which match given `title`.
+
+    Args:
+        icz (str): Identification number used to search Aleph.
+        base (str, optional): base on which will be search performed. Default
+                    :attr:`aleph.settings.ALEPH_DEFAULT_BASE`.
+
+    Returns:
+        int: Number of matching documents in Aleph.
+    """
+    return searchInAleph(base, icz, False, "icz")["no_entries"]
